
/*
 * ESP8266 SPIFFS HTML Web Page with JPEG, PNG Image 
 *
 */

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <FS.h>   //Include File System Headers

#include "ObslugaEEPROM.h"
#include "Formatowanie_SHTML.h"

//const char* imagefile = "/image.png";
//const char* htmlfile = "/index.html";

//ESP AP Mode configuration
char ssid[SIEC_NAZWA_ROZMIAR] = "image-demo-circuits4you.com";
char password[SIEC_HASLO_ROZMIAR] = "password";

//info statusu AP lub Stacja i IP
char status_IP[32];

ESP8266WebServer server(80);

void handleRoot(){
String path;
  //sprawdzenie czy SPIFFS przechowuje plik z rozszerzeniem index.shtml
  bool test = SPIFFS.exists("index.shtml");
  if (test ==true) path = "/index.shtml";
  else path = "/index.shtml";

  //server.sendHeader("Location", "/index.html",true);   //Redirect to our html web page
  server.sendHeader("Location", path,true);   //Redirect to our html web page
  server.send(302, "text/plane","");
}

void handleWebRequests()
{
String  message1 ="";
 for (uint8_t i=0; i<server.args(); i++)
 {
    message1 += server.argName(i) + "=" + server.arg(i) + "\n";
 }
 Obsluga_Komunikatu((char *)message1.c_str()); 

 message1 += "\n";
 Serial.println(message1);
  
  if(loadFromSpiffs(server.uri())) return;
  String message = "File Not Detected\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " NAME:"+server.argName(i) + "\n VALUE:" + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
  Serial.println(message);
}

void setup() {
int licznik;

  delay(1000);
  Serial.begin(115200);
  Serial.println();

  //Initialize File System
  SPIFFS.begin();
  Serial.println("File System SPIFFS zainicjowany");

  EEPROM_Odczyt_SSID(&ssid[0]);
  EEPROM_Odczyt_Hasla(&password[0]);
//  Serial.println(ssid);
//  Serial.println(password);
  
  //testowanie LINIA_WE4 ustawionej jako wejście
  //jeżeli po resecie stan niski wymuszenie trybu softAP
  pinMode(LINIA_WE1, INPUT_PULLUP);
  delay(100);
  int test_softAP =digitalRead(LINIA_WE1);
    
  if (test_softAP ==LOW)
  {
IPAddress local_IP(192,168,4,1);
IPAddress gateway(192,168,4,1);
IPAddress subnet(255,255,255,0);

  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(200);

    Serial.print("Setting soft-AP configuration ... ");
    Serial.println(WiFi.softAPConfig(local_IP, gateway, subnet) ? "Ready" : "Failed!");

    //Initialize AP Mode
    //WiFi.softAP("ARS3_miniAP","",1,4);  //Password not used
    WiFi.softAP("ARS3_miniAP");  //Password not used
    delay(100);
    IPAddress myIP = WiFi.softAPIP();
    Serial.print("Web Server ARS3_miniAP IP:");
    Serial.println(myIP);

    sprintf(status_IP, " AP IP:%u.%u.%u.%u", myIP[0],myIP[1],myIP[2],myIP[3]);
  }
  else
  {
    //inicjacja jako stacja
    Serial.print("Connecting to ");
    Serial.println(ssid);
     
    WiFi.begin(ssid, password);
    licznik =0;
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
      licznik++;
      if (licznik >10) break;
    }
    Serial.print("Web Server tryb Stacji IP: ");
    Serial.println(WiFi.localIP());   

    IPAddress myIP =WiFi.localIP();
    sprintf(status_IP, " Stacja IP:%u.%u.%u.%u", myIP[0],myIP[1],myIP[2],myIP[3]);
  }


  //Initialize Webserver
  server.on("/",handleRoot);
  server.onNotFound(handleWebRequests); //Set setver all paths are not found so we can handle as per URI
  server.on("/favicon.ico", []() {
    server.send(404, "text/plain", "");});
  server.begin();  

  //inicjacja wejsć/wyjsć binarnych
  Warunki_Tagow_IO_Ini();
}

void loop() {
 server.handleClient();
}

bool loadFromSpiffs(String path){
  String dataType = "text/plain";
  if(path.endsWith("/"))
  {
    //sprawdzenie czy SPIFFS przechowuje plik z rozszerzeniem index.shtml
    bool test = SPIFFS.exists("index.shtml");
    if (test ==true) path += "index.shtml";
    else path += "index.html";
  }

  if(path.endsWith(".src")) path = path.substring(0, path.lastIndexOf("."));
  else if(path.endsWith(".html")) dataType = "text/html";
  else if(path.endsWith(".htm")) dataType = "text/html";
  else if(path.endsWith(".shtml")) dataType = "text/html";
  else if(path.endsWith(".css")) dataType = "text/css";
  else if(path.endsWith(".js")) dataType = "application/javascript";
  else if(path.endsWith(".png")) dataType = "image/png";
  else if(path.endsWith(".gif")) dataType = "image/gif";
  else if(path.endsWith(".jpg")) dataType = "image/jpeg";
  else if(path.endsWith(".ico")) dataType = "image/x-icon";
  else if(path.endsWith(".xml")) dataType = "text/xml";
  else if(path.endsWith(".pdf")) dataType = "application/pdf";
  else if(path.endsWith(".zip")) dataType = "application/zip";
  File dataFile = SPIFFS.open(path.c_str(), "r");
  if (!dataFile) return false;
  if (server.hasArg("download")) dataType = "application/octet-stream";

  if(path.endsWith(".shtml"))
  {
    //plik jest typu .shtml
char *p_buf;
    p_buf =(char *)malloc(dataFile.size());
    if (p_buf !=NULL)
    {
      for(int i=0;i<dataFile.size();i++) //Read upto complete file size
      {
        //Serial.print((char)f.read());
        *(p_buf+i) =(char)dataFile.read();
      }
      /*
      char *poz =strstr(p_buf, "This");
      if (poz !=NULL)
      {
        memcpy(poz,"Ten przyklad",12);
      }
      */
      Procedura_Tagow_SHTML(p_buf);
      
      //server.client().write((const char*)p_buf, (size_t)strlen(p_buf));
      size_t rozmiar_pliku =dataFile.size();
      server.client().write((const char*)p_buf, rozmiar_pliku);
    }
    else
    {
      //nie powiodło sie przydzielenie pamięci  
      if (server.streamFile(dataFile, dataType) != dataFile.size()) { } 
    }
    free(p_buf);    
  }
  else 
  {
    //plik nie jest typu .shtml
    if (server.streamFile(dataFile, dataType) != dataFile.size()) { }
  }
  dataFile.close();

//  Serial.println(path.c_str());
//  Serial.println(dataType.c_str());
    
  return true;
}


